/**
 *
 * @module unserve-function
 */
(function () {
  'use strict'

  /** imports */
  var queryString = require('query-string')
  var request = require('request')
  var _ = require('lodash')
  var lomath = require('lomath')

  /** exports */
  module.exports = unserveFunction

  /**
   * Use a REST API as if it were a function
   *
   * @function unserveFunction
   * @alias unserve-function
   *
   * @param {String} url base address for the REST API
   * @return {Function} functional interface for the REST API at url
   */
  function unserveFunction (url) {
    return unserve

    /**
     * Use a REST API as if it were a function
     *
     * @function unserve
     *
     * @param {Object} options all function parameters
     * @param {Function} callback handle results
     */
    function unserve (options, callback) {
      if (options.data) {
        httpPost(url, options, callback)
      } else {
        httpGet(url, options, callback)
      }
    }
  }

  function httpGet (url, options, callback) {
    var query = '?' + queryString.stringify(options)

    url = url + query

    console.log('url: ' + url)

    request(url, function (error, response, body) {
      if (isProblem(error, response)) {
        error = handleProblem(error, response, body)
        callback(error, body)
        return
      }

      if (body) {
        body = JSON.parse(body)
      }

      callback(error, body)
    })
  }

  function isProblem (error, response) {
    return error || (response && response.statusCode !== 200)
  }

  function handleProblem (error, response, body) {
    var e = {}

    if (error) {
      e.error = error
    }

    if ((response && response.statusCode !== 200)) {
      e.statusCode = response.statusCode
    }

    return e
  }

  /**
   *
   * @param {String} url base address for the REST API
   * @param {Object} options function parameters
   * @param {*} options.data a large input parameter
   * @param {Function} callback handle results
   */
  function httpPost (url, options, callback) {
    var data = options.data

    options = _.omit(options, 'data')

    var query = '?' + queryString.stringify(options)

    url = url + query

    var requestOptions = {
      url: url,
      json: data
    }

    request.post(requestOptions, function (error, response, body) {
      error && console.error(error)

      if (typeof body === 'string') {
        body = lomath.unflattenJSON(body)
      }

      if (isProblem(error, response)) {
        error = handleProblem(error, response, body)
        callback(error, body)
        return
      }

      if (body) {
        body = lomath.unflattenJSON(body)
      }

      callback(error, body)
    })
  }
})()
